package com.dky.calculate;


import com.dky.utils.enums.EnergyPriceType;

public class MaterialKilnModel {

    // 标准煤转换系数：千克标准煤/千克
    private static final double coalConversionFactor = 0.7143;
    // 电能转换到标准煤的系数：千克标准煤/千瓦时
    private static final double electricityConversionFactor = 0.1229;
    // 标准煤转换到二氧化碳的系数：千克二氧化碳/千克标准煤
    private static final double carbonConversionFactor = 1.9003;

    /**
     * 计算初次投资费用
     *
     * @param productDensity       产品的密度
     * @param furnaceWorkingVolume 电窑炉的工作容积
     * @param expectedAnnualOutput 预计年产量
     * @param batchProductionHours 单台电设备制造一批产品的周期小时
     * @param equipmentAnnualHours 单台设备年运行时间
     * @param singleEquipmentPrice 单台电设备价格
     * @return 包含计算结果的对象
     */
    public static Double calculateProduction(
            Double productDensity,
            Double furnaceWorkingVolume,
            Double expectedAnnualOutput,
            Double batchProductionHours,
            Double equipmentAnnualHours,
            Double singleEquipmentPrice
    ) {
        // 单台电设备制造一批产品的产量 = 产品的密度*电窑炉的工作容积
        double singleBatchProduction = productDensity * furnaceWorkingVolume;

        // 电替代设备台数 = （预计年产量*单台电设备制造一批产品的周期小时<A>）/（单台电设备制造一批产品的产量<A>*单台设备年运行时间<B>）
        double equipmentCount = (expectedAnnualOutput * batchProductionHours) /
                (singleBatchProduction * equipmentAnnualHours);

        // 初次投资费用 = 电替代设备台数*单台电设备价格<A>
        return equipmentCount * singleEquipmentPrice;

    }


    /**
     * 计算年运行费用
     *
     * @param devicePower               电替代设备功率
     * @param deviceCount               电替代设备台数
     * @param deviceSubstituteLaborCost 电替代设备人工费用
     * @param deviceAnnualOperationTime 电替代设备年运行时间
     * @return 年运行费用
     */
    public static double calculateAnnualOperatingCost(Double devicePower, Integer deviceCount, Double deviceSubstituteLaborCost, Integer deviceAnnualOperationTime) {

        // 电替代设备年耗电量=单台电替代设备的功率<A>*电替代设备台数*电设备年运行时间<B>
        double annualElectricityConsumption = devicePower * deviceCount * deviceAnnualOperationTime;
        // 年运行费用=电替代设备年耗电量*工业平均电价(1.2元/千瓦时) + 单台电替代设备人工费用成本<A> * 台数
        return annualElectricityConsumption * EnergyPriceType.Industry_Electric_Price.getPrice() + deviceSubstituteLaborCost * deviceCount;

    }

    /**
     * 计算年总费用
     *
     * @param devicePower               电替代设备功率
     * @param deviceCount               电替代设备台数
     * @param deviceSubstituteLaborCost 电替代设备人工费用
     * @param deviceAnnualOperationTime 电替代设备年运行时间
     * @param devicePrice               设备价格
     * @param deviceServiceLife         设备使用年限
     * @return
     */
    public static double calculateAnnualTotalCost(Double devicePower, Integer deviceCount, Double deviceSubstituteLaborCost, Integer deviceAnnualOperationTime, Double devicePrice, Integer deviceServiceLife) {
        //（电替代设备台数*单台电设备价格<A>）/电替代设备的使用年限<A>
        double v = ((devicePrice * deviceCount) / deviceServiceLife);
        // 年总费用 =（电替代设备台数*单台电设备价格<A>）/电替代设备的使用年限<A> + 年运行费用
        return v + calculateAnnualOperatingCost(devicePower, deviceCount, deviceSubstituteLaborCost, deviceAnnualOperationTime);
    }

    /**
     * 计算年减碳量。
     *
     * @param previousYearOperatingCost 上年运行费用
     * @param originalEquipmentLaborCost 原设备的人工费用
     * @param electricitySubstitutionAnnualConsumption 电替代设备年耗电量
     * @return 年减碳量
     */
    public static double calculateAnnualCarbonReduction(
            Double previousYearOperatingCost,
            Double originalEquipmentLaborCost,
            Double electricitySubstitutionAnnualConsumption
    ) {
        // 计算节约的煤炭量（转换为标准煤）=（上年运行费用-原设备的人工费用）/煤炭平均价格(0.9元/千克)*0.7143千克标准煤/千克
        double savedCoalInStandardCoal = (previousYearOperatingCost - originalEquipmentLaborCost) /
                EnergyPriceType.Coal_Price_Per_Kilogram.getPrice() * coalConversionFactor;

        // 计算电替代设备消耗的电量转换为标准煤的量
        double electricityInStandardCoal = electricitySubstitutionAnnualConsumption * electricityConversionFactor;

        // 计算总的节约的标准煤量
        double totalSavedStandardCoal = savedCoalInStandardCoal - electricityInStandardCoal;

        // 计算年减碳量
        return totalSavedStandardCoal * carbonConversionFactor;
    }

    /**
     * 计算电替代设备年耗电量
     *
     * @param devicePower               设备功率
     * @param deviceCount               设备台数
     * @param deviceAnnualOperationTime 单台电设备年运行时间
     * @return
     */
    public static double calculateSubstituteElectricity(Double devicePower, Integer deviceCount, Integer deviceAnnualOperationTime) {
        // 电替代设备年耗电量 = 电替代设备的功率<A> * 电替代设备台数 * 电设备年运行时间<B>
        return devicePower * deviceCount * deviceAnnualOperationTime;

    }



}
